/*******************************************************************/
// The MIT License (MODIFIED)
//
// Copyright (c) 2002 Jake Bordens
//
// Permission is hereby granted, free of charge, to any person 
// obtaining a copy of  this software and associated documentation 
// files (the "Software"), to deal in the  Software without 
// restriction, including without limitation the rights to use,  
// copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the  Software, and to permit persons to whom the 
// Software is furnished to do so,  subject to the following 
// conditions:
//
// The above copyright notice and this permission notice shall be 
// included in all  copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
// OF MERCHANTABILITY, FITNESS  FOR A PARTICULAR PURPOSE AND 
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR  COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION  WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//
// Other Terms
// - If you chose to make a derivative work using the techniques 
//   found in this source code, a free license for the author is 
//   requested, but not required.
// - Including a copy of this license with your software is not
//   expressly required, so long as you assume all liability, and it 
//   is clear that the terms of the disclaimer apply to the author 
//   and/or copyright holder of this software in all circumstances.
/*******************************************************************/

#include <PalmOS.h>

//DialPhone function - By Jake B.
//DialPhone takes two parameters:
//
//    phoneNumber - a string.  Known valid characters: 1234567890-*#
//                  Known Invalid Charackters: A-Z
//
//    name        - also a string.  Displayed above the phone number
//                  in PhoneApp
//
void DialPhone(char* phoneNumber, char* name) {

   short* 		magicStruct;	//holds the magic block of memory that dials the phone
   int  		err;		//holds an error state
   int  		phoneDigits;	//StrLen of phone number

   DmSearchStateType	searchInfo;     //Used for finding the phone app.
   UInt16		cardNo = 0;
   LocalID		dbID;

   //I'm not exactly sure of the maximum phone number length.
   //I'm pretty sure that it MUST BE less than 64 bytes, but
   //you can experiment if you like.  I'm going with 32 to be
   //conservative.

   phoneDigits = StrLen(phoneNumber);
   if (phoneDigits < 32 && phoneDigits > 0) {
       magicStruct = MemPtrNew(214);    //The magicStruct is 214 bytes.
       MemSet(magicStruct, 214, 0);	//Clear it out to all 0's for good measure
					//(I don't know what everything is in there
					//so its better to be safe.)

       magicStruct[0x01] = 7; 		//This is a magic number that dials the phone
       magicStruct[0x27] = phoneDigits;	//0x27 holds the length of the phone number

       //Copy the phone number into the magic struct.   
       StrCopy((char*)&(magicStruct[0x28]), phoneNumber);

       if (name) {
	   int nameLen = StrLen(name);
           if (nameLen < 30) {
		magicStruct[0x49]=nameLen;
		StrCopy((char*)&magicStruct[0x4A],name);
           }	
       }

       //Find the PhoneApp's database and card number    
       err = DmGetNextDatabaseByTypeCreator (true, &searchInfo, 'appl',
                               'PhAp', true, &cardNo, &dbID);   

       if (!err) {
           //According to the PalmOS reference, you must set
           //the memory owner before passing it as a cmdPBP
           MemPtrSetOwner(magicStruct, 0);

           //Run the PhoneApp
           (void) SysUIAppSwitch (cardNo, dbID, sysAppLaunchCmdNormalLaunch, magicStruct);

	   //NOTE: I read somewhere that once the system (the '0'
           //in the MemPtrSetOwner above) has ownership of the
           //memory pointer, its no longer your responsibility to
           //free it.  I'm no expert.  It seems to work fine this
           //way, but it may be a memory leak.
       } else {
           //If we don't find the PhoneApp, free the magicStruct
           MemPtrFree(magicStruct);
       }
   }
}


//PilotMain - The entry point.
UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags) {
  int				err;


  if (cmd == sysAppLaunchCmdNormalLaunch) {
    EventType e;

    DialPhone("800-555-5555", "A Fake Number");

    //Wait until there's an appStopEvent
    do {
      EvtGetEvent (&e, evtWaitForever);
      SysHandleEvent (&e);
    } while (e.eType != appStopEvent);
     

  }
  return 0;

}
